/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.binding.http.rt;

import java.util.Map;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceAction.Direction;
import org.bluestemsoftware.specification.eoa.ext.connector.db.DataSourceConnector;

/**
 * Defines configuration used to configure a runtime <code>HTTPBinding</code> component
 * provider.
 */
public class HTTPBindingConfiguration {

    // initialize all instance vars with reasonable defaults

    private boolean embedStackTraceInFaults = true;
    private Map<QName, Integer> requestorTimeout;
    private int maxPayloadProcessors = 5;
    private String dataSourceConnectorName = DataSourceConnector.SYSTEM_DATA_SOURCE;
    private Map<QName, Map<String, String>> faultHeaderValues;
    private Map<QName, Map<Direction, Map<String,String>>> messageHeaderValues;

    /**
     * Gets amount of time before a blocking request times out, i.e the amount of time provider
     * will spend waiting on an anonymous response before throwing a fault.
     * 
     * @return value in milliseconds or -1 to indicate no timeout.
     */
    public int getRequestorTimeout(QName operationName) {
        int answer = 60000; // default to 1 minute
        if (requestorTimeout != null) {
            Integer integer = requestorTimeout.get(operationName);
            if (integer != null) {
                answer = integer.intValue();
            }
        }
        return answer;
    }

    public void setRequestorTimeout(Map<QName, Integer> requestorTimeout) {
        this.requestorTimeout = requestorTimeout;
    }
    
    /**
     * Returns value for an abstract http header declaration defined on corresponding binding
     * for indicated fault.
     * 
     * @param faultName
     * @param headerName
     * 
     * @return Header value or <code>null</code> if undefined.
     */
    public String getFaultHTTPHeaderValue(QName faultName, String headerName) {
        Map<String, String> headerValues = faultHeaderValues.get(faultName);
        if (headerValues == null) {
            return null;
        }
        return headerValues.get(headerName);
    }
    
    public void setFaultHeaderValues(Map<QName, Map<String, String>> faultHeaderValues) {
        this.faultHeaderValues = faultHeaderValues;
    }
    
    /**
     * Returns value for an abstract http header declaration defined on corresponding binding
     * for indicated message on indicated operation.
     * 
     * @param operationName
     * @param direction
     * @param headerName
     * @return Header value or <code>null</code> if undefined.
     */
    public String getMessageReferenceHTTPHeaderValue(QName operationName, Direction direction, String headerName) {
        Map<Direction, Map<String, String>> operationHeaders = messageHeaderValues.get(operationName);
        if (operationHeaders == null) {
            return null;
        }
        Map<String, String> directionHeaders = operationHeaders.get(direction);
        if (directionHeaders == null) {
            return null;
        }
        return directionHeaders.get(headerName);
    }
    
    public void setMessageHeaderValues(Map<QName, Map<Direction, Map<String, String>>> messageHeaderValues) {
        this.messageHeaderValues = messageHeaderValues;
    }

    /**
     * Defines DataSourceConnector name which MAY be required by binding provider to implement
     * message and/or header persistence. If element is undefined, the connector
     * 'SystemDataSource' is implied.
     * 
     * @return
     */
    public String getDataSourceConnectorName() {
        return dataSourceConnectorName;
    }
    
    public void setDataSourceConnectorName(String dataSourceConnectorName) {
        this.dataSourceConnectorName = dataSourceConnectorName;
    }

    /**
     * Defines maximum number of concurrent processors which process message payload, i.e.
     * which process action by binding it according to wsdl description and then passing to
     * engine.
     * 
     * @return
     */
    public int getMaxPayloadProcessors() {
        return maxPayloadProcessors;
    }

    public void setMaxPayloadProcessors(int maxPayloadProcessors) {
        this.maxPayloadProcessors = maxPayloadProcessors;
    }

    /**
     * Constrains fault handling.
     * 
     * @return true if stack trace embedded in detail. false if not.
     */
    public boolean isEmbeddStackTraceInFaults() {
        return embedStackTraceInFaults;
    }

    public void setEmbeddStackTraceInFaults(boolean embedStackTraceInFaults) {
        this.embedStackTraceInFaults = embedStackTraceInFaults;
    }

}
