/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.binding.soap;

import java.util.Map;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.component.binding.InterfaceReference;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceOperation;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceOperation.MEP;
import org.bluestemsoftware.specification.eoa.component.policy.rt.ElementPolicy;

public final class SOAP11Binding extends SOAPBinding {

    /**
     * Constructs a protocol specific, i.e. SOAP 11, <code>Binding</code> component.
     * 
     * @param name
     *        qualified name of <code>Binding</code> which MUST be unique within context of
     *        <code>Container</code>.
     * @param underlyingProtocol
     *        Models 'underlying' transmission protocol employed by <code>Binding</code>
     *        which MUST be consistent with soap version.
     * @param interfaceName
     *        Qualified name of Interface bound by Binding or <code>null</code> if binding is
     *        're-useable.'
     * @param elementPolicy
     *        Contains policy expressions which may be attached to <code>Binding</code>.
     * @param faults
     *        Collection of <code>BindingFault</code> components or an empty Collection if
     *        none defined or binding is re-useable.
     * @param operations
     *        Collection of <code>BindingOperation</code> components or an empty Collection
     *        if none defined or binding is re-useable.
     * 
     * @return Binding
     */
    public SOAP11Binding(ComponentName name, SOAPUnderlyingProtocol underlyingProtocol,
            InterfaceReference interfaceReference, ElementPolicy elementPolicy,
            Map<QName, SOAPBindingFault> faults, Map<QName, SOAPBindingOperation> operations) {
        super(name, underlyingProtocol, interfaceReference, elementPolicy, faults, operations, null);
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.binding.soap.SOAPBinding#getSoapVersion()
     */
    public SOAPVersion getSoapVersion() {
        return SOAPVersion.SOAP11;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.RootComponent#validate()
     */
    @Override
    protected void validate() throws DeploymentException {
        
        super.validate();
        
        // according to wsdl 20 soap 1.1 binding spec, the only supported abstract
        // meps are in-only and in-out
        
        if (!isReusable()) {
            for (InterfaceOperation io : interfaceReference.getReferencedComponent().getOperations()) {
                if (io.getMessageExchangePattern().equals(MEP.ROBUST_IN_ONLY)) {
                    throw new DeploymentException("MEP 'robust-in-only' not allowed for SOAP 1.1");
                }
            }
        } else {
            // must be validated by underlying protocol when provider validates
            // epr components
        }
        
    }

}
