/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.binding.soap;

import java.util.Map;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.component.binding.Binding;
import org.bluestemsoftware.specification.eoa.component.binding.BindingFault;
import org.bluestemsoftware.specification.eoa.component.binding.BindingOperation;
import org.bluestemsoftware.specification.eoa.component.binding.InterfaceReference;
import org.bluestemsoftware.specification.eoa.component.intrface.Interface;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceOperation;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceOperation.MEP;
import org.bluestemsoftware.specification.eoa.component.policy.rt.ElementPolicy;

/**
 * Models an extension to the core <code>Binding</code>, i.e. a <code>Binding</code> that
 * implements the SOAP messaging protocol.
 * 
 */
public abstract class SOAPBinding extends Binding {

    /**
     * URI which identifies WSDL binding type.
     */
    public static final String BINDING_TYPE = "http://www.w3.org/ns/wsdl/soap";
   
    /**
     * URI which identifies extension type (and provider namespace).
     */
    public static final String TYPE = "http://bluestemsoftware.org/specification/eoa/ext/binding/soap/default/1.0";
    
    /**
     * URI which identifies provider schema location (which defines provider namespace).
     */
    public static final String PROVIDER_SCHEMA_LOCATION = "classpath:///schema/http.bluestemsoftware.org.specification.eoa.ext.binding.soap.default.1.0.xsd";
    
    
    protected SOAPMessageExchangePattern soapMessageExchangePattern;

    public SOAPBinding(ComponentName name, SOAPUnderlyingProtocol underlyingProtocol,
            InterfaceReference interfaceReference, ElementPolicy elementPolicy,
            Map<QName, SOAPBindingFault> faults, Map<QName, SOAPBindingOperation> operations,
            SOAPMessageExchangePattern soapMessageExchangePattern) {
        super(BINDING_TYPE, name, underlyingProtocol, interfaceReference, elementPolicy, faults, operations);
        this.soapMessageExchangePattern = soapMessageExchangePattern;
    }

    /**
     * Defines default protocol specific message exchange pattern utilized by all operations in
     * scope or null if no assertion is made regarding default MEP.
     * <p>
     * If {@link SOAPVersion} employed by <code>Binding</code> is soap 1.1, no assertion is
     * made regarding soapMessageExchangePattern, i.e. <code>null</code> MUST be returned.
     * 
     * @return getSoapMessageExchangePattern
     */
    public SOAPMessageExchangePattern getSoapMessageExchangePattern() {
        return soapMessageExchangePattern;
    }

    /**
     * Indicates version of soap specification employed by binding.
     * 
     * @return SOAPVersion
     */
    public abstract SOAPVersion getSoapVersion();
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.binding.Binding#establishReferencesImpl()
     */
    @Override
    protected synchronized void establishReferences() throws DeploymentException {
        super.establishReferences();
        if (getFaults() != null) {
            for (BindingFault bindingFault : getFaults()) {
                SOAPMessage soapMessage = ((SOAPBindingFault)bindingFault).getSoapMessage();
                for (SOAPHeaderBlock soapHeaderBlock : soapMessage.getHeaders()) {
                    soapHeaderBlock.establishReference();
                }
            }
        }
        if (getOperations() != null) {
            SOAPBindingMessageReference sbmr = null;
            for (BindingOperation bindingOperation : getOperations()) {
                sbmr = (SOAPBindingMessageReference)bindingOperation.getInputMessageReference();
                if (sbmr != null) {
                    SOAPMessage soapMessage = sbmr.getSoapMessage();
                    for (SOAPHeaderBlock soapHeaderBlock : soapMessage.getHeaders()) {
                        soapHeaderBlock.establishReference();
                    }
                }
                sbmr = (SOAPBindingMessageReference)bindingOperation.getOutputMessageReference();
                if (sbmr != null) {
                    SOAPMessage soapMessage = sbmr.getSoapMessage();
                    for (SOAPHeaderBlock soapHeaderBlock : soapMessage.getHeaders()) {
                        soapHeaderBlock.establishReference();
                    }
                }
            }
        }
        if (getUnderlyingProtocol() != null) {
            SOAPUnderlyingProtocol underlyingProtocol = (SOAPUnderlyingProtocol)getUnderlyingProtocol();
            if (underlyingProtocol instanceof SOAP12HTTPProtocol) {
                ((SOAP12HTTPProtocol)underlyingProtocol).establishReferences();
            }
        }
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.RootComponent#validate()
     */
    @Override
    protected void validate() throws DeploymentException {
        
        super.validate();
        ((SOAPUnderlyingProtocol)underlyingProtocol).validate();
        
        // if binding is interface specific, verify that soap mep is consistent
        // with WSDL MEP as mapped by spec. note that SOAP mep is undefined for
        // SOAP 11 bindings
        
        if (!isReusable() && getSoapVersion() == SOAPVersion.SOAP12) {
            Interface intrface = getInterfaceReference().getReferencedComponent();
            for (InterfaceOperation io : intrface.getOperations()) {
                MEP wsdlMEP = io.getMessageExchangePattern();
                BindingOperation bo = getOperation(io.getName());
                if (bo != null) {
                    SOAPBindingOperation sbo = (SOAPBindingOperation)bo;
                    SOAPMessageExchangePattern soapMEP = sbo.getSoapMessageExchangePattern();
                    if (soapMEP == null) {
                        soapMEP = getSoapMessageExchangePattern();
                    }
                    if (soapMEP == null) {
                        if (wsdlMEP != MEP.IN_OUT) {
                            throw new DeploymentException("Definition for SOAP 1.2 Binding "
                                    + getName()
                                    + " is invalid. SOAP MEP is undefined for operation "
                                    + bo.getName()
                                    + " and binding defines no default MEP. WSDL MEP must be 'In-Out'.");
                        }                        
                    }
                    if (wsdlMEP == MEP.IN_ONLY || wsdlMEP == MEP.ROBUST_IN_ONLY) {
                        if (soapMEP == SOAPMessageExchangePattern.SOAP_12_SOAP_RESPONSE) {
                            throw new DeploymentException("Definition for SOAP 1.2 Binding "
                                    + getName()
                                    + " is invalid. SOAP MEP "
                                    + soapMEP
                                    + " is not consistent with WSDL MEP "
                                    + wsdlMEP
                                    + ". Expected "
                                    + SOAPMessageExchangePattern.SOAP_12_REQUEST_RESPONSE
                                    + ".");
                        }
                    } else {                        
                        // wsdl mep is in-out. both soap meps are valid, i.e. if soap mep
                        // soap-response, the request is serialized to request uri. else
                        // it's a straight mapping of wsdl request response to soap request
                        // response
                    }
                }
            }
        }
        
        // TODO: validate http header refs a simple schema type
    }

    // *********************** Identity Methods ************************************

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("name: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(name);
        sb.append(System.getProperty("line.separator"));
        sb.append("bindingType: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(BINDING_TYPE);
        sb.append(System.getProperty("line.separator"));
        sb.append("interfaceReference: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(interfaceReference);
        sb.append(System.getProperty("line.separator"));
        sb.append("underlyingProtocol: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(underlyingProtocol);
        sb.append(System.getProperty("line.separator"));
        sb.append("soapVersion: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(getSoapVersion());
        sb.append(System.getProperty("line.separator"));
        sb.append("soapMessageExchangePattern: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(soapMessageExchangePattern);
        sb.append(System.getProperty("line.separator"));
        sb.append("faults:");
        sb.append(System.getProperty("line.separator"));
        sb.append(faults);
        sb.append(System.getProperty("line.separator"));
        sb.append("operations:");
        sb.append(System.getProperty("line.separator"));
        sb.append(operations);
        return sb.toString();
    }

    @Override
    public int hashCode() {
        return hashCode(getBindingType())
                + hashCode(name)
                + hashCode(interfaceReference)
                + hashCode(underlyingProtocol)
                + hashCode(getSoapVersion())
                + hashCode(soapMessageExchangePattern)
                + hashCode(getFaults())
                + hashCode(getOperations());
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }

        if (!(obj instanceof SOAPBinding)) {
            return false;
        }

        if ((getBindingType() == ((SOAPBinding)obj).getBindingType() || (getBindingType() != null && getBindingType().equals(((SOAPBinding)obj)
                .getBindingType())))
                && (name == ((SOAPBinding)obj).name || (name != null && name.equals(((SOAPBinding)obj).name)))
                && (interfaceReference == ((SOAPBinding)obj).interfaceReference || (interfaceReference != null && interfaceReference
                        .equals(((SOAPBinding)obj).interfaceReference)))
                && (underlyingProtocol == ((SOAPBinding)obj).underlyingProtocol || (underlyingProtocol != null && underlyingProtocol
                        .equals(((SOAPBinding)obj).underlyingProtocol)))
                && (getSoapVersion() == ((SOAPBinding)obj).getSoapVersion() || (getSoapVersion() != null && getSoapVersion()
                        .equals(((SOAPBinding)obj).getSoapVersion())))
                && (soapMessageExchangePattern == ((SOAPBinding)obj).soapMessageExchangePattern || (soapMessageExchangePattern != null && soapMessageExchangePattern
                        .equals(((SOAPBinding)obj).soapMessageExchangePattern)))
                && (faults == ((SOAPBinding)obj).faults || (faults != null && faults
                        .equals(((SOAPBinding)obj).faults)))
                && (operations == ((SOAPBinding)obj).operations || (operations != null && operations
                        .equals(((SOAPBinding)obj).operations)))) {
            return true;
        }

        return false;
    }

    private int hashCode(Object object) {
        return (object == null ? 0 : object.hashCode());
    }

}
