/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.binding.soap.rt;

import java.util.Arrays;
import java.util.List;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.intrface.rt.ApplicationFault;
import org.bluestemsoftware.specification.eoa.component.intrface.rt.FaultContext;
import org.bluestemsoftware.specification.eoa.component.message.rt.Content;
import org.bluestemsoftware.specification.eoa.component.message.rt.ContentFactory;
import org.bluestemsoftware.specification.eoa.ext.binding.soap.SOAPVersion;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Converts an {@link ApplicationFault} response returned by a locally hosted service into a
 * SOAPReceiverFault, i.e. message direction is 'Out' and MEP is in-out or robust-in-only.
 * <p>
 * A compliant <code>Binding</code> MUST use the implied code, subcode, reason and detail to
 * return a soap fault to client.
 */
public class SOAPApplicationFault extends SOAPReceiverFault {

    private static final long serialVersionUID = 1L;

    protected String faultAction;
    protected Content faultMessage;

    public SOAPApplicationFault(SOAPVersion soapVersion, FaultContext faultContext) {
        super(soapVersion, faultContext.getFault().getFaultName(), ((Throwable)faultContext.getFault())
                .getMessage());
        faultAction = faultContext.getAction();
        faultMessage = faultContext.getMessage().getContent();
        if (faultMessage == null) {
            throw new IllegalArgumentException("fault message content null");
        }
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.binding.soap.rt.SOAPReceiverFault#appendFaultDetail(org.bluestemsoftware.specification.eoa.component.message.rt.Content)
     */
    @Override
    public void appendFaultDetail(Content faultEnvelope) {
        ContentFactory contentFactory = faultEnvelope.getFactory();
        Node imported = null;
        try {
            imported = contentFactory.importContent(faultMessage);
        } catch (UnsupportedOperationException ue) {
            imported = contentFactory.importNode(faultMessage, true);
        }
        Element faultDetail = getFaultDetailElement(faultEnvelope);
        faultDetail.appendChild(imported);
    }

    @Override
    public QName getFaultCode() {
        if (soapVersion == SOAPVersion.SOAP11) {
            return faultName;
        } else {
            return SOAP_12_FAULT_CODE;
        }
    }

    @Override
    public String getAction() {
        return faultAction;
    }

    @Override
    public List<QName> getFaultSubCodes() {
        return Arrays.asList(faultName);
    }

}
