/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.connector.db;

import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.bluestemsoftware.specification.eoa.system.container.Connector;
import org.bluestemsoftware.specification.eoa.system.container.Container;

public class DataSourceConnector extends Connector implements DataSource {

    public static final String TYPE = DataSourceConnector.class.getName();

    /**
     * Name used to identify 'system' datasource, i.e. a 'default' connector which MAY be
     * relied upon by <code>Extension</code> providers deployed within context of
     * <code>System</code> to provide their behavior.
     * 
     * @see Container#getSystemDataSource()
     */
    public static final String SYSTEM_DATA_SOURCE = "SystemDataSource";

    protected Dialect dialect;
    protected Method unwrapMethod;
    protected Method isWrapperForMethod;

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends Connector.Provider, DataSource {
        public Dialect spi_getDialect();
    }

    public static enum Dialect {
        DB2,
        DERBY,
        POSTGRE_SQL,
        MY_SQL,
        ORACLE,
        SYBASE,
        SQL_SERVER,
        SAP_DB,
        INFORMIX,
        HSQLDB,
        INGRES,
        PROGRESS,
        MCKOI,
        INTERBASE,
        POINTBASE,
        FRONTBASE,
        FIREBIRD,
        UNKNOWN
    }

    public DataSourceConnector(DataSourceConnectorFactory factory, Connector.Provider provider, String name) {
        super(factory, provider, name);
        try {
            Class<?> clazz = DataSource.class;
            Class<?>[] signature = { Class.class };
            unwrapMethod = clazz.getDeclaredMethod("unwrap", signature);
        } catch (NoSuchMethodException ignore) {
        }
        try {
            Class<?> clazz = DataSource.class;
            Class<?>[] signature = { Class.class };
            isWrapperForMethod = clazz.getDeclaredMethod("isWrapperFor", signature);
        } catch (NoSuchMethodException ignore) {
        }
    }

    /**
     * Returns dialect understood by this DataSource instance.
     * 
     * @return Dialect
     */
    public Dialect getDataSourceDialect() {
        if (dialect == null) {
            Thread thread = Thread.currentThread();
            ClassLoader cl = thread.getContextClassLoader();
            try {
                thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
                dialect = ((Provider)provider).spi_getDialect();
            } finally {
                thread.setContextClassLoader(cl);
            }
        }
        return dialect;
    }

    /** *************************** DataSource Impl ************************** * */

    /*
     * (non-Javadoc)
     * @see javax.sql.DataSource#getConnection()
     */
    public Connection getConnection() throws SQLException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).getConnection();
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /*
     * (non-Javadoc)
     * @see javax.sql.DataSource#getConnection(java.lang.String, java.lang.String)
     */
    public Connection getConnection(String username, String password) throws SQLException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).getConnection(username, password);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /*
     * (non-Javadoc)
     * @see javax.sql.DataSource#getLogWriter()
     */
    public PrintWriter getLogWriter() throws SQLException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).getLogWriter();
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /*
     * (non-Javadoc)
     * @see javax.sql.DataSource#getLoginTimeout()
     */
    public int getLoginTimeout() throws SQLException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).getLoginTimeout();
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /*
     * (non-Javadoc)
     * @see javax.sql.DataSource#setLogWriter(java.io.PrintWriter)
     */
    public void setLogWriter(PrintWriter out) throws SQLException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            ((Provider)provider).setLogWriter(out);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /*
     * (non-Javadoc)
     * @see javax.sql.DataSource#setLoginTimeout(int)
     */
    public void setLoginTimeout(int seconds) throws SQLException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            ((Provider)provider).setLoginTimeout(seconds);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * JDK 1.6 DataSource method. Implemented using reflection API which allows
     * specification-eoa-ext to be compiled using JDK 1.5 in cases where underlying
     * dataSource(s) unavailable for JDK 1.6.
     * 
     * @see java.sql.Wrapper#isWrapperFor(java.lang.Class)
     */
    public boolean isWrapperFor(Class<?> iface) throws SQLException {
        if (isWrapperForMethod != null) {
            Thread thread = Thread.currentThread();
            ClassLoader cl = thread.getContextClassLoader();
            try {
                thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
                return (Boolean)isWrapperForMethod.invoke(provider, new Object[] { iface });
            } catch (Exception ex) {
                throw new SQLException(ex.toString());
            } finally {
                thread.setContextClassLoader(cl);
            }
        } else {
            return false;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * JDK 1.6 DataSource method. Implemented using reflection API which allows
     * specification-eoa-ext to be compiled using JDK 1.5 in cases where underlying
     * dataSource(s) unavailable for JDK 1.6.
     * 
     * @see java.sql.Wrapper#unwrap(java.lang.Class)
     */
    public <T> T unwrap(Class<T> iface) throws SQLException {
        if (unwrapMethod != null) {
            Thread thread = Thread.currentThread();
            ClassLoader cl = thread.getContextClassLoader();
            try {
                thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
                return iface.cast(unwrapMethod.invoke(provider, new Object[] { iface }));
            } catch (Exception ex) {
                throw new SQLException(ex.toString());
            } finally {
                thread.setContextClassLoader(cl);
            }
        }
        return null;
    }

}
