/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.container.standalone10;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.Resource;
import org.bluestemsoftware.specification.eoa.ext.container.ContainerFactory;
import org.bluestemsoftware.specification.eoa.ext.container.standalone10.StandaloneContainerConfiguration.ConnectorInfo;
import org.bluestemsoftware.specification.eoa.ext.container.standalone10.StandaloneContainerConfiguration.PoolingSupportType;
import org.bluestemsoftware.specification.eoa.ext.container.standalone10.StandaloneContainerConfiguration.TransactionSupportInfo;
import org.bluestemsoftware.specification.eoa.ext.container.standalone10.StandaloneContainerConfiguration.TransactionSupportType;
import org.bluestemsoftware.specification.eoa.ext.container.standalone10.StandaloneContainerConfiguration.ConnectorInfo.ConnectorTransactionSupportInfo;
import org.bluestemsoftware.specification.eoa.ext.container.standalone10.StandaloneContainerConfiguration.ConnectorInfo.PoolingSupportInfo;
import org.bluestemsoftware.specification.eoa.system.container.Container;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * A factory for producing a 'standalone' <code>Container</code>.
 */

public final class StandaloneContainerFactory extends ContainerFactory {

    public static final String NAME = "standalone-container";

    public interface Provider extends ContainerFactory.Provider {
        public Container spi_createContainer(StandaloneContainerFactory factory, StandaloneContainerConfiguration config) throws StandaloneContainerException;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.ExtensionFactory#getExtensionType()
     */
    public final String getExtensionType() {
        return StandaloneContainer.TYPE;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.container.ContainerFactory#createContainer(javax.xml.transform.dom.DOMSource)
     */
    @Override
    public Container createContainer(DOMSource domSource) throws StandaloneContainerException {
        if (factoryContext == null) {
            throw new IllegalStateException("Factory has no context.");
        }
        if (domSource == null) {
            throw new IllegalArgumentException("domSource null");
        }
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factoryContext.getClassLoader());
            validateDefinition(domSource);
            Element definition = (Element)domSource.getNode();
            StandaloneContainerConfiguration config = new StandaloneContainerConfiguration();
            config.setTransactionSupportInfo(createTransactionSupportInfo(definition));
            config.setConnectorInfos(createConnectorInfos(definition));
            return ((Provider)provider).spi_createContainer(this, config);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    private TransactionSupportInfo createTransactionSupportInfo(Element definition) throws StandaloneContainerException {

        TransactionSupportInfo transactionSupportInfo = null;

        QName childName = new QName(StandaloneContainer.TYPE, "transactionSupport");
        Element transactionSupportElement = getChildElement(definition, childName);

        if (transactionSupportElement != null) {
            transactionSupportInfo = new TransactionSupportInfo();
            childName = new QName(StandaloneContainer.TYPE, "defaultTransactionTimeoutSeconds");
            Element timeoutElement = getChildElement(transactionSupportElement, childName);
            if (timeoutElement != null) {
                transactionSupportInfo.setDefaultTransactionTimeoutSeconds(Integer
                        .parseInt(getText(timeoutElement)));
            }
        }

        return transactionSupportInfo;
    }

    private Set<ConnectorInfo> createConnectorInfos(Element definition) throws StandaloneContainerException {

        Set<ConnectorInfo> connectorInfos = new HashSet<ConnectorInfo>();

        QName childName = new QName(StandaloneContainer.TYPE, "connectors");
        Element connectorsElement = getChildElement(definition, childName);

        if (connectorsElement != null) {

            List<Element> connectorElements = getChildElements(connectorsElement, new QName(
                    StandaloneContainer.TYPE, "connector"));

            for (Element connectorElement : connectorElements) {

                ConnectorInfo connectorInfo = new ConnectorInfo();

                connectorInfo.setType(connectorElement.getAttribute("type"));
                connectorInfo.setImpl(connectorElement.getAttribute("impl"));
                connectorInfo.setConnectorName(connectorElement.getAttribute("name"));

                childName = new QName(StandaloneContainer.TYPE, "properties");
                Element propertiesElement = getChildElement(connectorElement, childName);
                connectorInfo.setProperties(new Properties());
                List<Element> propertyElements = getChildElements(propertiesElement, new QName(
                        StandaloneContainer.TYPE, "property"));
                for (Element propertyElement : propertyElements) {
                    String propertyName = propertyElement.getAttribute("name");
                    String propertyValue = getText(propertyElement);
                    if (propertyValue == null) {
                        propertyValue = "";
                    }
                    connectorInfo.getProperties().put(propertyName, propertyValue);
                }

                childName = new QName(StandaloneContainer.TYPE, "connectionSupport");
                Element connectionSupportElement = getChildElement(connectorElement, childName);

                ConnectorTransactionSupportInfo transactionSupportInfo = new ConnectorTransactionSupportInfo();
                connectorInfo.setTransactionSupportInfo(transactionSupportInfo);
                childName = new QName(StandaloneContainer.TYPE, "transactionSupport");
                Element transactionSupportElement = getChildElement(connectionSupportElement, childName);
                String supportTypeStr = transactionSupportElement.getAttribute("type");
                transactionSupportInfo.setTransactionSupportType(TransactionSupportType.valueOf(supportTypeStr));

                childName = new QName(StandaloneContainer.TYPE, "useTransactionCaching");
                Element useTransactionCachingElement = getChildElement(transactionSupportElement, childName);
                if (useTransactionCachingElement != null) {
                    String useTransactionCachingStr = getText(useTransactionCachingElement);
                    transactionSupportInfo
                            .setUseTransactionCaching(Boolean.parseBoolean(useTransactionCachingStr));
                }
                childName = new QName(StandaloneContainer.TYPE, "useThreadCaching");
                Element useThreadCachingElement = getChildElement(transactionSupportElement, childName);
                if (useThreadCachingElement != null) {
                    String useThreadCachingStr = getText(useThreadCachingElement);
                    transactionSupportInfo.setUseThreadCaching(Boolean.parseBoolean(useThreadCachingStr));
                }

                PoolingSupportInfo poolingSupportInfo = new PoolingSupportInfo();
                connectorInfo.setPoolingSupportInfo(poolingSupportInfo);
                childName = new QName(StandaloneContainer.TYPE, "poolingSupport");
                Element poolingSupportElement = getChildElement(connectionSupportElement, childName);
                String poolingTypeStr = poolingSupportElement.getAttribute("type");
                poolingSupportInfo.setPoolingSupportType(PoolingSupportType.valueOf(poolingTypeStr));

                childName = new QName(StandaloneContainer.TYPE, "minPoolSize");
                Element minPoolSizeElement = getChildElement(poolingSupportElement, childName);
                if (minPoolSizeElement != null) {
                    poolingSupportInfo.setMinPoolSize(Integer.parseInt(getText(minPoolSizeElement)));
                }
                childName = new QName(StandaloneContainer.TYPE, "maxPoolSize");
                Element maxPoolSizeElement = getChildElement(poolingSupportElement, childName);
                if (maxPoolSizeElement != null) {
                    poolingSupportInfo.setMaxPoolSize(Integer.parseInt(getText(maxPoolSizeElement)));
                }
                childName = new QName(StandaloneContainer.TYPE, "blockingTimeoutMilliseconds");
                Element blockingTimeoutMillisecondsElement = getChildElement(poolingSupportElement, childName);
                if (blockingTimeoutMillisecondsElement != null) {
                    poolingSupportInfo.setBlockingTimeoutMilliseconds(Integer
                            .parseInt(getText(blockingTimeoutMillisecondsElement)));
                }
                childName = new QName(StandaloneContainer.TYPE, "idleTimeoutMilliseconds");
                Element idleTimeoutMillisecondsElement = getChildElement(poolingSupportElement, childName);
                if (idleTimeoutMillisecondsElement != null) {
                    poolingSupportInfo.setIdleTimeoutMilliseconds(Integer
                            .parseInt(getText(idleTimeoutMillisecondsElement)));
                }
                childName = new QName(StandaloneContainer.TYPE, "matchOne");
                Element matchOneElement = getChildElement(poolingSupportElement, childName);
                poolingSupportInfo.setMatchOne(Boolean.parseBoolean(getText(matchOneElement)));
                childName = new QName(StandaloneContainer.TYPE, "matchAll");
                Element matchAllElement = getChildElement(poolingSupportElement, childName);
                poolingSupportInfo.setMatchAll(Boolean.parseBoolean(getText(matchAllElement)));
                childName = new QName(StandaloneContainer.TYPE, "selectOneAssumeMatch");
                Element selectOneAssumeMatchElement = getChildElement(poolingSupportElement, childName);
                poolingSupportInfo.setSelectOneAssumeMatch(Boolean
                        .parseBoolean(getText(selectOneAssumeMatchElement)));

                if (poolingSupportInfo.getPoolingSupportType() == PoolingSupportType.PARTITIONED) {
                    childName = new QName(StandaloneContainer.TYPE, "partitionByConnectionRequest");
                    Element partitionByConnectionRequestElement = getChildElement(poolingSupportElement, childName);
                    if (partitionByConnectionRequestElement == null) {
                        throw new StandaloneContainerException("Error configuring pooling support for connector '"
                                + connectorInfo.getConnectorName()
                                + "'. Support PARTITIONED, requires 'partitionByConnectionRequest' element");
                    }
                    poolingSupportInfo.setPartitionByConnectionRequest(Boolean
                            .parseBoolean(getText(partitionByConnectionRequestElement)));
                    childName = new QName(StandaloneContainer.TYPE, "partitionBySubject");
                    Element partitionBySubjectElement = getChildElement(poolingSupportElement, childName);
                    if (partitionBySubjectElement == null) {
                        throw new StandaloneContainerException("Error configuring pooling support for connector '"
                                + connectorInfo.getConnectorName()
                                + "'. Support PARTITIONED, requires 'partitionBySubject' element");
                    }
                    poolingSupportInfo.setPartitionBySubject(Boolean
                            .parseBoolean(getText(partitionBySubjectElement)));
                }

                connectorInfos.add(connectorInfo);
            }

        }
        return connectorInfos;

    }

    private List<Element> getChildElements(Element parent, QName childName) {
        List<Element> answer = new ArrayList<Element>();
        NodeList nodeList = parent.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element element = (Element)nodeList.item(i);
            if (childName.getNamespaceURI().equals(XMLConstants.NULL_NS_URI)) {
                if (element.getNamespaceURI() != null) {
                    continue;
                }
            } else {
                if (element.getNamespaceURI() == null) {
                    continue;
                }
                if (!childName.getNamespaceURI().equals("*")
                        && !element.getNamespaceURI().equals(childName.getNamespaceURI())) {
                    continue;
                }
            }
            if (!childName.getLocalPart().equals("*") && !element.getLocalName().equals(childName.getLocalPart())) {
                continue;
            }
            answer.add(element);
        }
        return answer;
    }

    private Element getChildElement(Element parent, QName childName) {
        NodeList nodeList = parent.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element element = (Element)nodeList.item(i);
            if (childName.getNamespaceURI().equals(XMLConstants.NULL_NS_URI)) {
                if (element.getNamespaceURI() != null) {
                    continue;
                }
            } else {
                if (element.getNamespaceURI() == null) {
                    continue;
                }
                if (!childName.getNamespaceURI().equals("*")
                        && !element.getNamespaceURI().equals(childName.getNamespaceURI())) {
                    continue;
                }
            }
            if (!childName.getLocalPart().equals("*") && !element.getLocalName().equals(childName.getLocalPart())) {
                continue;
            }
            return element;
        }
        return null;
    }

    private void validateDefinition(DOMSource source) throws StandaloneContainerException {

        Resource resource = null;
        try {
            resource = factoryContext.getResource(StandaloneContainer.PROVIDER_SCHEMA_LOCATION);
        } catch (DeploymentException de) {
            throw new StandaloneContainerException("Error validating definition. " + de);
        }

        SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

        javax.xml.validation.Schema schema = null;
        try {
            schema = schemaFactory.newSchema(new StreamSource(resource.getInputStream()));
        } catch (Exception ex) {
            throw new StandaloneContainerException("Error parsing schema. " + ex);
        }
        DOMValidator domValidator = new DOMValidator(schema);
        String validationError;
        try {
            validationError = domValidator.validate(source);
        } catch (Exception ex) {
            throw new StandaloneContainerException("Error validating definition. " + ex);
        }
        if (validationError != null) {
            throw new StandaloneContainerException("Engine definition invalid. " + validationError);
        }

    }

    static final class DOMValidator {

        private Schema schema;

        public DOMValidator(Schema schema) {
            this.schema = schema;
        }

        public String validate(DOMSource source) throws SAXException, IOException {
            Validator validator = schema.newValidator();
            validator.setErrorHandler(new ErrorHandlerImpl());
            validator.validate(source);
            return ((ErrorHandlerImpl)validator.getErrorHandler()).getErrors();
        }

        static class ErrorHandlerImpl implements ErrorHandler {

            private ArrayList<String> errorMessages = new ArrayList<String>();

            public void error(SAXParseException spe) throws SAXException {
                errorMessages.add(spe.getMessage());
            }

            public void warning(SAXParseException spe) throws SAXException {
                errorMessages.add(spe.getMessage());
            }

            public void fatalError(SAXParseException spe) throws SAXException {
                errorMessages.add(spe.getMessage());
            }

            public String getErrors() {
                if (errorMessages.size() > 0) {
                    Iterator<String> messageIterator = errorMessages.iterator();
                    StringBuilder errorMessage = new StringBuilder();
                    while (messageIterator.hasNext()) {
                        errorMessage.append(messageIterator.next() + "  ");
                    }
                    return errorMessage.toString();
                }
                return null;
            }

        }

    }

    private String getText(Element element) {
        if (element == null || !element.hasChildNodes()) {
            return null;
        }
        try {
            Text textNode = (Text)element.getFirstChild();
            return textNode.getData();
        } catch (ClassCastException ce) {
        }
        return null;
    }

}
