/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.auth;

public final class HostInfo {

    /**
     * Proxy for this 'machines' host name which MAY be used within policy etc ... to identify
     * 'my' host.
     */
    public static final String MY_HOST = "myhost";

    private String host;
    private int port;
    private String serializedForm;
    
    /**
     * Constructor which implies value '-1' for port, i.e. applies to 
     * all ports on indicated host.
     * 
     * @param host
     */
    public HostInfo(String host) {
        this(host, -1);
    }

    /**
     * Constructor which allows specification of port.
     * 
     * @param host
     *        name of host
     * @param port
     *        port or '0' which implies all ports on indicated host.
     */
    public HostInfo(String host, int port) {
        if (host == null || host.equals("")) {
            throw new IllegalArgumentException("Host null or empty string");
        }
        if (port < -1) {
            throw new IllegalArgumentException("Port must be >= -1");
        }
        this.host = host;
        this.port = port;
    }

    /**
     * Gets name of host.
     * 
     * @return
     */
    public String getHost() {
        return host;
    }

    /**
     * Gets port for this host.
     * 
     * @return port or '-1' which implies all ports for this host.
     */
    public int getPort() {
        return port;
    }

    // *********************** Identity Methods ************************************

    @Override
    public final String toString() {
        if (serializedForm == null) {
            serializedForm = host + (port > -1 ? ":" + port : "");
        }
        return serializedForm;
    }

    @Override
    public int hashCode() {
        return host.hashCode() + new Integer(port).hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof HostInfo)) {
            return false;
        }
        if (host.equals(((HostInfo)obj).host) && (port == ((HostInfo)obj).port)) {
            return true;
        }
        return false;
    }

}
