/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.jmx.server;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.bluestemsoftware.specification.eoa.ext.ManageableExtension;
import org.bluestemsoftware.specification.eoa.system.server.Feature;

public class JMXServerFeature extends Feature {

    public static final String TYPE = JMXServerFeature.class.getName();

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends Feature.Provider {
        public MBeanServer spi_getMBeanServer();
        public ObjectName spi_registerMBean(ManageableExtension manageableExtension, Object mbean, String type, String name) throws JMXServerFeatureException;
        public ObjectName spi_registerSystemMBean(Object mbean, String type, String name) throws JMXServerFeatureException;
        public void spi_unRegisterMBean(ObjectName objectName) throws JMXServerFeatureException;
    }
    
    public JMXServerFeature(JMXServerFeatureFactory factory, Feature.Provider provider) {
        super(factory, provider);
    }

    /**
     * Gets <code>MBeanServer</code> created and registered by this feature.
     * 
     * @return MBeanServer configured according to feature specific configuration
     */
    public MBeanServer getMBeanServer() {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).spi_getMBeanServer();
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /**
     * Registers an <code>MBean</code> which MAY be used to manage a specific
     * <code>Extension</code> and returns name generated by feature which MUST be unique
     * within context of <code>System</code>.
     * 
     * @param manageableExtension
     *        instance exposing management interface
     * @param mbean
     * @param type
     *        key property 'type' used to construct object name, i.e. represents a grouping of
     *        'name' properties. A <code>null</code> value MAY be passed if all mbeans for
     *        <code>ManageableExtension</code> are of same type, i.e. no subgroups.
     * @param name
     *        key property 'name' used to construct object name. Value SHOULD be simple name of
     *        mbean interface class
     * 
     * @return ObjectName mbean name generated by feature
     * @throws JMXServerFeatureException if an error occurs registering mbean
     */
    public ObjectName registerMBean(ManageableExtension manageableExtension, Object mbean, String type, String name) throws JMXServerFeatureException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).spi_registerMBean(manageableExtension, mbean, type, name);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /**
     * Registers an <code>MBean</code> which MAY be used to manage <code>System</code> and
     * returns name generated by feature which MUST be unique within context of
     * <code>System</code>.
     * 
     * @param mbean
     * @param type
     *        key property 'type' used to construct object name, i.e. represents a grouping of
     *        'name' properties. A <code>null</code> value MAY be passed if all system mbeans
     *        are of same type, i.e. no subgroups.
     * @param name
     *        key property 'name' used to construct object name. Value SHOULD be simple name of
     *        mbean interface class
     * 
     * @return ObjectName mbean name generated by feature
     * @throws JMXServerFeatureException if an error occurs registering mbean
     * 
     */
    public ObjectName registerSystemMBean(Object mbean, String type, String name) throws JMXServerFeatureException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).spi_registerSystemMBean(mbean, type, name);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }
    
    /**
     * Unregisters a previoulsy registered <code>MBean</code>.
     * 
     * @param objectName
     * @throws JMXServerFeatureException if an error occurs unregistering mbean
     */
    public void unRegisterMBean(ObjectName objectName) throws JMXServerFeatureException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            ((Provider)provider).spi_unRegisterMBean(objectName);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

}
