/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing;

import java.util.Collections;
import java.util.Set;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.engine.rt.FeatureModule.WSSpecification;
import org.bluestemsoftware.specification.wsa.EndpointReference;

public abstract class WSA implements WSSpecification {

    public static WSA getInstance(String namespace) {
        if (namespace.equals(WSA10.NAMESPACE_URI)) {
            return new WSA10();
        } else {
            return null;
        }
    }

    public static final String NAMESPACE_PREFIX = "wsa";
    public static final String TO = "To";
    public static final String FROM = "From";
    public static final String REPLY_TO = "ReplyTo";
    public static final String FAULT_TO = "FaultTo";
    public static final String ENDPOINT_REFERENCE = "EndpointReference";
    public static final String ADDRESS = "Address";
    public static final String ACTION = "Action";
    public static final String MESSAGE_ID = "MessageID";
    public static final String RELATES_TO = "RelatesTo";
    public static final String REF_PARMS = "ReferenceParameters";
    public static final String RELATIONSHIP_TYPE = "RelationshipType";

    public interface Policy extends WSSpecification.Policy {
        public interface Infoset extends WSSpecification.Infoset {
        }
    }

    public static class WSA10 extends WSA {

        public static final String NAMESPACE_URI = "http://www.w3.org/2005/08/addressing";
        public static final String ANONYMOUS_URI = "http://www.w3.org/2005/08/addressing/anonymous";
        public static final String NONE_URI = "http://www.w3.org/2005/08/addressing/none";
        public static final String UNSPECIFIED_MESSAGE = "http://www.w3.org/2005/08/addressing/unspecified";
        public static final String DEFAULT_RELATIONSHIP_TYPE = "http://www.w3.org/2005/08/addressing/reply";
        public static final EndpointReference NONE_REPLY_TO = new EndpointReference(NONE_URI);
        public static final EndpointReference DEFAULT_REPLY_TO = new EndpointReference(ANONYMOUS_URI);

        public static class Infoset implements WSSpecification.Infoset {

            public static final QName TO_ELEM = new QName(NAMESPACE_URI, TO, NAMESPACE_PREFIX);
            public static final QName FROM_ELEM = new QName(NAMESPACE_URI, FROM, NAMESPACE_PREFIX);
            public static final QName REPLY_TO_ELEM = new QName(NAMESPACE_URI, REPLY_TO, NAMESPACE_PREFIX);
            public static final QName FAULT_TO_ELEM = new QName(NAMESPACE_URI, FAULT_TO, NAMESPACE_PREFIX);
            public static final QName ENDPOINT_REFERENCE_ELEM = new QName(NAMESPACE_URI, ENDPOINT_REFERENCE, NAMESPACE_PREFIX);
            public static final QName ADDRESS_ELEM = new QName(NAMESPACE_URI, ADDRESS, NAMESPACE_PREFIX);
            public static final QName ACTION_ELEM = new QName(NAMESPACE_URI, ACTION, NAMESPACE_PREFIX);
            public static final QName MESSAGE_ID_ELEM = new QName(NAMESPACE_URI, MESSAGE_ID, NAMESPACE_PREFIX);
            public static final QName RELATES_TO_ELEM = new QName(NAMESPACE_URI, RELATES_TO, NAMESPACE_PREFIX);
            public static final QName REF_PARMS_ELEM = new QName(NAMESPACE_URI, REF_PARMS, NAMESPACE_PREFIX);
            public static final QName RELATIONSHIP_TYPE_ATT = new QName(RELATIONSHIP_TYPE);

        }

        public static class Policy implements WSA.Policy {

            public static final String PRIVATE_PVN = "http://bluestemsoftware.org/specification/eoa/ext/feature/ws/addressing/policy/1.0";
            public static final String PRIVATE_PVN_PREFIX = "wsaf";
            public static final String PVN = "http://www.w3.org/2007/05/addressing/metadata";
            public static final String PVN_PREFIX = "wsam";

            private Boolean isAnonymousResponses;
            private String replyToAddress;

            public Policy(Boolean isAnonymousResponses) {
                this.isAnonymousResponses = isAnonymousResponses;
            }

            public Policy(Boolean isAnonymousResponses, String replyToAddress) {
                this.isAnonymousResponses = isAnonymousResponses;
                this.replyToAddress = replyToAddress;
            }

            /**
             * Translates a wsam:Addressing assertion into a Boolean value. Note that a
             * compliant <code>BindingModule</code> MUST enforce that expression contains
             * exactly one non-empty, alternative, i.e. exactly one alternative containing an
             * 'Addressing' assertion with namespace ({@link #PVN}, which implies that policy
             * set retrieved via {@link WSA#getPolicy()} will contain exactly one entry.
             * 
             * @return Boolean True if anonymous responses required. False if non-anonymous
             *         response required. Null if no assertion made regarding anonymous
             *         responses, i.e. 'client' decides using an unspecified algorithm.
             */
            public Boolean isAnonymousResponses() {
                return isAnonymousResponses;
            }

            /**
             * Gets wsa:ReplyTo address. If {@link #isAnonymousResponses()} is false, returns
             * value which MUST be used by <code>Binding</code> to set wsa:ReplyTo. If
             * {@link #isAnonymousResponses()} is <code>null</code> and replyTo address is
             * undefined, <code>Binding</code> MUST set wsa:ReplyTo to
             * {@link WSA10#ANONYMOUS_URI}. If {@link #isAnonymousResponses()} is
             * <code>null</code> and replyTo address is defined, <code>Binding</code> MUST
             * set wsa:ReplyTo to the returned value. If {@link #isAnonymousResponses()} is
             * true, <code>Binding</code> MUST set wsa:ReplyTo to {@link WSA10#ANONYMOUS_URI}
             * 
             * @return
             */
            public String getReplyToAddress() {
                return replyToAddress;
            }

            public static class Infoset implements WSA.Policy.Infoset {
                public static final QName REPLY_TO_ATT = new QName(PRIVATE_PVN, "replyTo", PRIVATE_PVN_PREFIX);
                public static final QName ADDRESSING_ELEM = new QName(PVN, "Addressing");
                public static final QName ANONYMOUS_RESPONSES_ELEM = new QName(PVN, "AnonymousResponses");
                public static final QName NONANONYMOUS_RESPONSES_ELEM = new QName(PVN, "NonAnonymousResponses");
            }

            public Infoset getInfoset() {
                return new Infoset() {
                };
            }

        }

        public WSA10() {
        }

        public WSA10(Set<WSSpecification.Policy> alternatives) {
            this.alternatives = alternatives;
        }

        private Set<WSSpecification.Policy> alternatives;

        @Override
        public WSSpecification.Infoset getInfoSet() {
            return new Infoset() {
            };
        }

        @Override
        public String getNamespace() {
            return NAMESPACE_URI;
        }

        @Override
        @SuppressWarnings("unchecked")
        public Set<WSSpecification.Policy> getPolicy() {
            if (alternatives == null) {
                return Collections.EMPTY_SET;
            }
            return alternatives;
        }

    }

    public abstract Infoset getInfoSet();

    public abstract String getNamespace();

    public abstract Set<WSSpecification.Policy> getPolicy();

}
