/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.binding.BindingFault;
import org.bluestemsoftware.specification.eoa.component.binding.BindingFaultReference;
import org.bluestemsoftware.specification.eoa.component.binding.BindingMessageReference;
import org.bluestemsoftware.specification.eoa.component.binding.BindingOperation;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceAction.Direction;
import org.bluestemsoftware.specification.eoa.component.policy.rt.PolicyExpression;
import org.bluestemsoftware.specification.eoa.ext.binding.http.HTTPBinding;
import org.bluestemsoftware.specification.eoa.ext.binding.http.XMLHTTPProtocol;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.http.HTTPHeader;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.http.HTTPLocationInfo;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.http.HTTPMethod;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.WSDLException;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.BindingOperationOutfaultElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.DescriptionElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document;
import org.bluestemsoftware.specification.eoa.ext.policy.PolicyException;
import org.w3c.dom.Element;

public class HTTPBindingWriter {

    public static void writeBinding(WSDL20Document wsdlDocument, HTTPBinding binding) throws WSDLException {

        HTTPBinding httpBinding = (HTTPBinding)binding;

        DescriptionElement description = wsdlDocument.getDescriptionElement();

        // create the binding element
        HTTPBindingElement bindingElement = wsdlDocument.createHTTPBindingElement();
        description.addBindingElement(bindingElement);
        bindingElement.setName(binding.getName().getLocalPart());
        bindingElement.setType(HTTPBinding.BINDING_TYPE);

        if (binding.getInterfaceReference() != null) {
            bindingElement.setInterface(binding.getInterfaceReference().getReferencedComponentName());
        }

        if (binding.getElementPolicy() != null) {
            PolicyExpression policy;
            try {
                policy = binding.getElementPolicy().getMergedExpression();
            } catch (PolicyException pe) {
                throw new WSDLException("Error writing description. " + pe);
            }
            writePolicy(policy, bindingElement);
        }

        // retrieve underlying protocol

        XMLHTTPProtocol xmlHTTPProtocol = (XMLHTTPProtocol)httpBinding.getUnderlyingProtocol();

        // note. our reader converts encoding default and separator default for
        // binding to message ref granularity. so no need to set here

        bindingElement.setMethodDefault(xmlHTTPProtocol.getMethodDefault());
        bindingElement.setUseHTTPCookies(xmlHTTPProtocol.useHTTPCookies());
        
        // iterate over binding faults and create a binding fault element for each

        for (BindingFault bindingFault : httpBinding.getFaults()) {

            HTTPBindingFaultElement bindingFaultElement = wsdlDocument.createHTTPBindingFaultElement();
            bindingElement.addFaultElement(bindingFaultElement);
            bindingFaultElement.setRef(bindingFault.getFaultName());

            for (HTTPHeader httpHeader : xmlHTTPProtocol.getFaultHTTPHeaders(bindingFault.getFaultName())) {
                HTTPHeaderElement httpHeaderElement = wsdlDocument.createHTTPHeaderElement();
                bindingFaultElement.addHTTPHeaderElement(httpHeaderElement);
                httpHeaderElement.setName(httpHeader.getName());
                httpHeaderElement.setType(httpHeader.getType());
            }
            
            String tcoding = xmlHTTPProtocol.getFaultHTTPContentEncoding(bindingFault.getFaultName());
            if (tcoding != null) {
                bindingFaultElement.setContentEncoding(tcoding);
            }
            
            String errorStatusCodeValue = null;
            Integer errorStatusCode = xmlHTTPProtocol.getErrorStatusCode(bindingFault.getFaultName());
            if (errorStatusCode != null) {
                errorStatusCodeValue = errorStatusCode.toString();
            } else {
                errorStatusCodeValue = "#any";
            }
            bindingFaultElement.setErrorStatusCode(errorStatusCodeValue);

            if (bindingFault.getElementPolicy() != null) {
                PolicyExpression policy;
                try {
                    policy = bindingFault.getElementPolicy().getMergedExpression();
                } catch (PolicyException pe) {
                    throw new WSDLException("Error writing description. " + pe);
                }
                writePolicy(policy, bindingFaultElement);
            }

        }

        for (BindingOperation bindingOperation : binding.getOperations()) {

            QName operationName = bindingOperation.getName();

            HTTPBindingOperationElement bindingOperationElement = wsdlDocument.createHTTPBindingOperationElement();
            bindingElement.addOperationElement(bindingOperationElement);
            bindingOperationElement.setRef(operationName);

            if (bindingOperation.getElementPolicy() != null) {
                PolicyExpression policy;
                try {
                    policy = bindingOperation.getElementPolicy().getMergedExpression();
                } catch (PolicyException pe) {
                    throw new WSDLException("Error writing description. " + pe);
                }
                writePolicy(policy, bindingOperationElement);
            }

            // note. our reader converts encoding default for operation to message ref
            // granularity, so no need to set here

            String separator = xmlHTTPProtocol.getHTTPQueryParameterSeparator(operationName);
            if (separator != null) {
                bindingOperationElement.setQueryParameterSeparator(separator);
            }
            
            HTTPMethod httpMethod = xmlHTTPProtocol.getMethod(operationName);
            if (httpMethod != null) {
                bindingOperationElement.setMethod(httpMethod);
            }
            
            HTTPLocationInfo locationInfo = xmlHTTPProtocol.getHTTPLocationInfo(operationName);
            if (locationInfo != null) {
                bindingOperationElement.setLocation(locationInfo.getLocation());
                bindingOperationElement.setIgnoreUncited(locationInfo.isIgnoreUncited());
            }
            
            String inputSerialization = xmlHTTPProtocol.getInputSerialization(operationName);
            if (inputSerialization != null) {
                bindingOperationElement.setInputSerialization(inputSerialization);
            }
            
            String outputSerialization = xmlHTTPProtocol.getOutputSerialization(operationName);
            if (outputSerialization != null) {
                bindingOperationElement.setOutputSerialization(outputSerialization);
            }
            
            String faultSerialization = xmlHTTPProtocol.getFaultSerialization(operationName);
            if (faultSerialization != null) {
                bindingOperationElement.setFaultSerialization(faultSerialization);
            }
            
            // ************* process binding input **************************

            BindingMessageReference inputMessageReference = bindingOperation.getInputMessageReference();

            HTTPBindingOperationInputElement inputElement = wsdlDocument.createHTTPBindingOperationInputElement();
            bindingOperationElement.addInputElement(inputElement);

            if (inputMessageReference.getElementPolicy() != null) {
                PolicyExpression policy;
                try {
                    policy = inputMessageReference.getElementPolicy().getMergedExpression();
                } catch (PolicyException pe) {
                    throw new WSDLException("Error writing description. " + pe);
                }
                writePolicy(policy, inputElement);
            }

            for (HTTPHeader httpHeader : xmlHTTPProtocol.getMessageReferenceHTTPHeaders(operationName,
                    Direction.IN)) {
                HTTPHeaderElement httpHeaderElement = wsdlDocument.createHTTPHeaderElement();
                inputElement.addHTTPHeaderElement(httpHeaderElement);
                httpHeaderElement.setName(httpHeader.getName());
                httpHeaderElement.setType(httpHeader.getType());
            }
            String tcoding = xmlHTTPProtocol.getMessageReferenceHTTPContentEncoding(operationName, Direction.IN);
            if (tcoding != null) {
                inputElement.setContentEncoding(tcoding);
            }

            // ************* process binding output **************************

            BindingMessageReference outputMessageReference = bindingOperation.getOutputMessageReference();

            if (outputMessageReference != null) {

                HTTPBindingOperationOutputElement outputElement = wsdlDocument
                        .createHTTPBindingOperationOutputElement();
                bindingOperationElement.addOutputElement(outputElement);

                if (outputMessageReference.getElementPolicy() != null) {
                    PolicyExpression policy;
                    try {
                        policy = outputMessageReference.getElementPolicy().getMergedExpression();
                    } catch (PolicyException pe) {
                        throw new WSDLException("Error writing description. " + pe);
                    }
                    writePolicy(policy, outputElement);
                }

                if (xmlHTTPProtocol != null) {
                    for (HTTPHeader httpHeader : xmlHTTPProtocol.getMessageReferenceHTTPHeaders(operationName,
                            Direction.OUT)) {
                        HTTPHeaderElement httpHeaderElement = wsdlDocument.createHTTPHeaderElement();
                        outputElement.addHTTPHeaderElement(httpHeaderElement);
                        httpHeaderElement.setName(httpHeader.getName());
                        httpHeaderElement.setType(httpHeader.getType());
                    }
                    tcoding = xmlHTTPProtocol.getMessageReferenceHTTPContentEncoding(operationName, Direction.IN);
                    if (tcoding != null) {
                        outputElement.setContentEncoding(tcoding);
                    }
                }

            }

            for (BindingFaultReference bindingFaultReference : bindingOperation.getFaultReferences()) {
                BindingOperationOutfaultElement faultReferenceElement = wsdlDocument
                        .createBindingOperationOutfaultElement();
                faultReferenceElement.setRef(bindingFaultReference.getFault().getFaultName());
                if (bindingFaultReference.getElementPolicy() != null) {
                    PolicyExpression policy;
                    try {
                        policy = bindingFaultReference.getElementPolicy().getMergedExpression();
                    } catch (PolicyException pe) {
                        throw new WSDLException("Error writing description. " + pe);
                    }
                    writePolicy(policy, faultReferenceElement);
                }
            }

        }

    }

    public static void writePolicy(PolicyExpression policyExpression, Element parent) throws WSDLException {
        parent.appendChild(parent.getOwnerDocument().adoptNode(policyExpression));
    }

}
