/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.intrface;

import java.util.List;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceOperation.Style;

/**
 * Models content of an IRI Style compliant element declaration which conforms with rules
 * defined by WSDL 2.0 adjuncts specification.
 */
public class IRIStyle implements Style {

    public static final String STYLE_IRI = "http://www.w3.org/ns/wsdl/style/iri";

    private QName elementName;
    private List<ChildElement> childElements;

    /**
     * Constructs an <code>IRIStyle</code> instance.
     * 
     * @param elementName
     *        Qualified name of 'wrapper' element, the local part of which MUST match name of
     *        operation upon which style is defined.
     * 
     * @param childElements
     *        local element children which MUST be listed in order in which they are declared
     *        and which MUST be typed as an xs:simpleType and MUST NOT be of the type or derive
     *        from xs:QName, xs:NOTATION, xs:hexBinary or xs:base64Binary.
     * 
     */
    public IRIStyle(QName elementName, List<ChildElement> childElements) {
        if (elementName == null) {
            throw new IllegalArgumentException("elementName null");
        }
        if (childElements == null) {
            throw new IllegalArgumentException("childElements null");
        }
        this.elementName = elementName;
        this.childElements = childElements;
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.intrface.InterfaceOperation$Style#getStyleIRI()
     */
    public String getStyleIRI() {
        return STYLE_IRI;
    }

    /**
     * Gets local element children listed in order in which they were declared.
     * 
     * @return element children or an empty list if none defined.
     */
    public List<ChildElement> getChildElements() {
        return childElements;
    }

    /**
     * Gets qualified name of 'wrapper' element, the local part of which MUST match name of
     * operation upon which style is defined.
     * 
     * @return
     */
    public QName getElementName() {
        return elementName;
    }

    /**
     * Models a local child element declaration.
     * 
     */
    public static class ChildElement {

        private boolean nillable;
        private String name;
        private int minOccurs;
        private String defaultValue;

        /**
         * Constructs <code>ChildElement</code> instance.
         * 
         * @param name local name of child element.
         * @param nillable
         *        true if nillable, false if not. If true, defaultValue MUST be null.
         * @param minOccurs
         * @param defaultValue
         *        default value used when instance of child element not found in instance data.
         *        MUST be null if nillable is true.
         */
        public ChildElement(String name, boolean nillable, int minOccurs, String defaultValue) {
            if (name == null) {
                throw new IllegalArgumentException("name null");
            }
            this.name = name;
            this.nillable = nillable;
            this.minOccurs = minOccurs;
            this.defaultValue = defaultValue;
        }

        public String getName() {
            return name;
        }

        public boolean isNillable() {
            return nillable;
        }

        public String getDefaultValue() {
            return defaultValue;
        }

        public int getMinOccurs() {
            return minOccurs;
        }

    }

}
